import argparse
from typing import List

from .types import APP
from .app import Slack


def main(argv: List[str] | None = None):
    parser = argparse.ArgumentParser(prog="alert")
    parser.add_argument("message", type=str, default="alert")
    parser.add_argument("--app", type=APP, default=APP.SLACK, choices=list(APP))
    parser.add_argument("--verbose", action="store_true")
    parser.add_argument("--disable", action="store_true")
    args, _ = parser.parse_known_args(argv)

    if args.app == APP.SLACK:
        caller = Slack(verbose=args.verbose)
    else:
        raise NotImplementedError(
            f"Notice for {args.app} has not been implemented."
        )

    if not args.disable:
        caller(args.message)


if __name__ == "__main__":
    raise SystemExit(main())


from enum import Enum


class APP(str, Enum):
    SLACK = "slack"


class AT(str, Enum):
    BOTH = "both"
    START = "start"
    END = "end"


class ON(str, Enum):
    ALWAYS = "always"
    SUCCESS = "success"
    ERROR = "error"


import traceback
from typing import Callable, Any

from .types import AT, ON, APP
from .app import Slack


class alert:
    def __init__(
        self,
        message: str,
        at: AT = AT.BOTH,
        on: ON = ON.ALWAYS,
        app: APP = APP.SLACK,
        verbose: bool = False,
        disable: bool = False,
    ):
        self.message = message
        self.at = at
        self.on = on
        self.app = app
        self.verbose = verbose
        self.disable = disable

        if self.app == APP.SLACK:
            self.caller = Slack(verbose=self.verbose)
        else:
            raise NotImplementedError(
                f"Notice for {self.app} has not been implemented."
            )

    # Context manager
    def __enter__(self):
        if self.at in (AT.BOTH, AT.START) and not self.disable:
            message = "START: " + self.message
            self.caller(message)
        return self

    def __exit__(self, exc_type, exc, tb):
        if (self.at in (AT.BOTH, AT.END) or exc) and not self.disable:
            # replace message with traceback if exception is occurred
            if exc:
                message = "ERROR: " + "".join(traceback.format_exception(exc_type, exc, tb))
            else:
                message = "END: " + self.message

            if (
                self.on == ON.ALWAYS
                or (self.on == ON.SUCCESS and exc is None)
                or (self.on == ON.ERROR and exc)
            ):
                self.caller(message)
        return False

    # Decorator
    def __call__(self, func: Callable[..., Any]) -> Callable[..., Any]:
        def wrapper(*args, **kwargs):
            with self:  # context manager를 내부에서 재사용
                return func(*args, **kwargs)

        return wrapper



import os
import requests

from .base import BaseApp


class Slack(BaseApp):
    def _alert(self, text):
        url = os.environ.get("SLACK_WEBHOOK_URL", None)
        requests.post(url, json={"text": text})



class BaseApp:
    def __init__(self, verbose: bool):
        self.verbose = verbose

    def __call__(self, text: str):
        try:
            self._alert(text)
            if self.verbose:
                print(f"successfully called {self} alarm")
        except Exception as e:
            if self.verbose:
                print(f"failed to call {self} alarm: {e}")

    def _alert(self, text: str):
        raise NotImplementedError

    def __repr__(self):
        return "Base"
